# Timers
# Part of Dolby 1.1
# Taken from XiRCON's default.tcl

set timers(0) ""
unset timers(0)

alias timer* {

    set id [string range [lindex [args] 0] 5 end]
    set args [lrange [args] 1 end]

    if { ![string length $id] || ![string compare [string tolower $id] s] } {
        set id 0
    }

    set off 0
    set beg 0

    set i [lsearch $args -o]
    if { $i < 0 } { set i [lsearch $args -O] }
    if { $i >= 0 } {
        set off 1
        set args [lreplace $args $i $i]
    }

    set i [lsearch $args *:*]
    if { $i >= 0 } {
        catch { set beg [clock scan [lindex $args $i]] }
        set args [lreplace $args $i $i]
    }

    set rep [lindex $args 0]
    set int [lindex $args 1]
    set cmd [join [lrange $args 2 end]]

    if { ![string compare [string tolower $rep] off] } {
        kill_timer $id
    } elseif { [llength $args] >= 3 } {
        make_timer $id $off $beg $rep $int $cmd
    } else {
        show_timers
    }

    complete
}

on timer {
    set names [array names timers]
    foreach id $names {
        process_timer $id
    }
}

proc make_timer { id off beg rep int cmd } {
    global timers
    if { ![string compare $id 0] } { set id [get_timer_id] }
    set timers($id) [list $off $beg $rep $int $cmd 0 0]
    echo "*** Timer$id activated"
}

proc kill_timer { id } {
    global timers
    if { [catch { unset timers($id) }] } {
        echo "*** Timer$id is not active"
    } else {
        echo "*** Timer$id halted"
    }
}

proc show_timers { } {
    global timers
    set names [array names timers]
    set count 0
    foreach id $names {
        set t $timers($id)
        set o "*** Timer$id"
        if { [lindex $t 1] > 0 } {
            append o " [clock format [lindex $t 1] -format %H:%M:%S]"
        }
        append o " [lindex $t 2]x"
        append o " [lindex $t 3]s"
        append o " [lindex $t 4]"
        echo $o
        incr count
    }
    if { !$count } { echo "*** No active timers" }
}

proc get_timer_id { } {
    global timers
    set i 1
    set names [array names timers]
    while { [lsearch $names $i] >= 0 } { incr i }
    return $i
}

proc process_timer { id } {
    global timers
    set ct  [clock seconds]
    set off [lindex $timers($id) 0]
    set beg [lindex $timers($id) 1]
    set rep [lindex $timers($id) 2]
    set int [lindex $timers($id) 3]
    set cmd [lindex $timers($id) 4]
    set lr  [lindex $timers($id) 5]
    set lt  [lindex $timers($id) 6]

    if { ![connected] && !$off } {
        return
    }
    if { $ct < $beg } {
        return
    }
    if { $ct < [expr $lt + $int] } {
        return
    }

    set beg 0

    incr lr

    set lt $ct

    eval $cmd

    if { $lr == $rep } {
        kill_timer $id
        return
    }

    set timers($id) [list $off $beg $rep $int $cmd $lr $lt]
}

